from tfmock.db import Database

access_token = "eyJ0eXAiOiJKV1QiLCJhbGciOiJSUzI1NiIsImtpZCI6Imk2bEdrM0ZaenhSY1ViMkMzbkVRN3N5SEpsWSJ9" \
               ".eyJhdWQiOiI2ZTc0MTcyYi1iZTU2LTQ4NDMtOWZmNC1lNjZhMzliYjEyZTMiLCJpc3MiOiJodHRwczovL2xvZ2" \
               "luLm1pY3Jvc29mdG9ubGluZS5jb20vNzJmOTg4YmYtODZmMS00MWFmLTkxYWItMmQ3Y2QwMTFkYjQ3L3YyLjAiLCJpYX" \
               "QiOjE1MzcyMzEwNDgsIm5iZiI6MTUzNzIzMTA0OCwiZXhwIjoxNTM3MjM0OTQ4LCJhaW8iOiJBWFFBaS84SUFBQUF0QWFaTG8z" \
               "Q2hNaWY2S09udHRSQjdlQnE0L0RjY1F6amNKR3hQWXkvQzNqRGFOR3hYZDZ3TklJVkdSZ2hOUm53SjFsT2NBbk5aY2p2a295ckZ" \
               "4Q3R0djMzMTQwUmlvT0ZKNGJDQ0dWdW9DYWcxdU9UVDIyMjIyZ0h3TFBZUS91Zjc5UVgrMEtJaWpkcm1wNjlSY3R6bVE9PSIsImF6cC" \
               "I6IjZlNzQxNzJiLWJlNTYtNDg0My05ZmY0LWU2NmEzOWJiMTJlMyIsImF6cGFjciI6IjAiLCJuYW1lIjoiQWJlIExpbmNvbG4iLCJvaWQ" \
               "iOiI2OTAyMjJiZS1mZjFhLTRkNTYtYWJkMS03ZTRmN2QzOGU0NzQiLCJwcmVmZXJyZWRfdXNlcm5hbWUiOiJhYmVsaUBtaWNyb3NvZnQuY29" \
               "tIiwicmgiOiJJIiwic2NwIjoiYWNjZXNzX2FzX3VzZXIiLCJzdWIiOiJIS1pwZmFIeVdhZGVPb3VZbGl0anJJLUtmZlRtMjIyWDVyclYzeERxZkt" \
               "RIiwidGlkIjoiNzJmOTg4YmYtODZmMS00MWFmLTkxYWItMmQ3Y2QwMTFkYjQ3IiwidXRpIjoiZnFpQnFYTFBqMGVRYTgyUy1JWUZBQSIsInZlciI6IjIuMCJ9.pj4N-w_3Us9DrBLfpCt"


class MsiMock:
    def handle_msi(self, urlmap, req, req_body):
        if urlmap.path == '/mock_msi':
            return {'status': 200, 'reason': '', 'headers': {'content-type': 'application/json'}, 'body': {
                "access_token": access_token,
                "refresh_token": "",
                "expires_in": "3599",
                "expires_on": "1506484173",
                "not_before": "1506480273",
                "resource": "https://management.azure.com/",
                "token_type": "Bearer"
            }}



import json
import sqlite3


def get_connect():
    return sqlite3.connect('mock.db')


# noinspection PyShadowingBuiltins
class Database:
    def __init__(self):
        con = get_connect()
        cur = con.cursor()
        cur.execute('CREATE TABLE IF NOT EXISTS RESOURCES(ID, PAYLOAD)')
        con.close()

    def save(self, id, payload):
        con = get_connect()
        cur = con.cursor()
        if isinstance(payload, bytes):
            json_str = payload.decode('utf-8')
        elif isinstance(payload, str):
            json_str = payload
        else:
            json_str = json.dumps(payload)

        if self.exist(id):
            cur.execute('UPDATE RESOURCES SET PAYLOAD = ? WHERE ID = ?', (json_str, id.lower()))
        else:
            cur.execute('INSERT INTO RESOURCES(ID, PAYLOAD) VALUES(?,?)', (id.lower(), json_str))

        con.commit()
        con.close()

    def exist(self, id):
        con = get_connect()
        cur = con.cursor()
        res = cur.execute('SELECT ID FROM RESOURCES WHERE ID = ?', (id.lower(),))
        flag = res.fetchone() is not None
        con.close()
        return flag

    def get(self, id):
        con = get_connect()
        cur = con.cursor()
        res = cur.execute('SELECT PAYLOAD FROM RESOURCES WHERE ID = ?', (id.lower(),))
        payload = json.loads(res.fetchone()[0])
        con.close()
        return payload

    def query(self, id):
        with get_connect() as con:
            cur = con.cursor()
            res = cur.execute('SELECT ID, PAYLOAD FROM RESOURCES WHERE ID LIKE ?', (f'{id.lower()}/%',))
            return res.fetchall()

    def delete(self, id):
        with get_connect() as con:
            cur = con.cursor()
            cur.execute('DELETE FROM RESOURCES WHERE ID = ?', (id.lower(),))



import json
import re

from tfmock.db import Database


class VirtualMachineScaleSetMock:
    database = Database()

    def handle_vmss(self, urlmap, req, req_body):
        match = re.match(r'^/subscriptions/((\w|-)+)/resourceGroups/((\w|-)+)/providers/Microsoft.Compute/virtualMachineScaleSets/((\w|-)+)$', urlmap.path, re.IGNORECASE)
        if match is None:
            return

        res_id = urlmap.path
        name = res_id.split('/')[-1]

        if req.command == 'GET':
            if not self.database.exist(res_id):
                return {'status': 404, 'reason': '', 'headers': {}, 'body': {}}

            return {'status': 200, 'reason': '', 'headers': {'content-type': 'application/json'}, 'body': self.database.get(res_id)}

        if req.command == 'PUT' or req.command == 'PATCH':
            body = json.loads(req_body)
            self.database.save(res_id, {
                **body,
                'id': res_id,
                'name': name,
                'provisioningState': 'Succeeded'
            })
            return {'status': 200, 'reason': '', 'headers': {'content-type': 'application/json'}, 'body': self.database.get(res_id)}

        if req.command == 'DELETE':
            self.database.delete(res_id)
            return {'status': 200, 'reason': '', 'headers': {'content-type': 'application/json'}, 'body': {}}



import logging
from urllib.parse import urlsplit

from tfmock.cloud_services.azure.services import list_mock_classes


class AzureMocker:
    def __init__(self):
        self.chains = []
        for name, service in list_mock_classes():
            for method_name in dir(service):
                if method_name.startswith('handle_'):
                    self.chains.append(eval(f'service().{method_name}'))

    def mock_request(self, req, req_body):
        urlmap = urlsplit(req.path)
        for chain in self.chains:
            res = chain(urlmap, req, req_body)
            if res is not None:
                return res

        logging.warning(f'fail process {req.command} {req.path}')

        return {
            'status': 200,
            'reason': 'created',
            'headers': {
                'content-type': 'application/json'
            },
            'body': {}
        }



import importlib
import inspect
import os

ms = []

for file in os.listdir(os.path.dirname(__file__)):
    mod_name = file[:-3]
    if not mod_name.startswith('__'):
        ms.append(importlib.import_module(f'.{mod_name}', 'cloud_services.azure.services'))


def list_mock_classes():
    for m in ms:
        for name, cls in inspect.getmembers(m):
            if not name.startswith('__') and inspect.isclass(cls):
                yield name, cls
